//      utility functions

//      note that date routines have NOT been checked for Y2K compliance!


STRING  st_cpy( char dest[], char src[], int maxchar )
/*
        Copy a string from "src" to "dest".  At the most, "maxchar-1" characters
        will be copied, in order to accomodate the terminating '\0'.  "dest" must
        have been declared/allocated "maxchar" characters long.
*/
{
    register    int     n ;
    register    char    *d ;
    int         l ;

    d = dest ;
    l = maxchar - 1 ;
    n = strlen( src );
    if ( n > l )
        n = l ;
    memmove( d, src, n );
    dest[n] = '\0' ;
    return( d );                                // destination string
}


STRING  st_cat( char dest[], char src[], int maxchar )
/*
        Append a string from "src" to "dest".  At the most, ("maxchar"-1) characters will be copied, in order to
        accomodate the terminating NULL.  "dest" must be "maxchar" characters long.
*/
{
    int         n, p ;

    p = strlen( dest );
    n = maxchar - p ;
    if ( n > 0 )
        st_cpy( &dest[p], src, n );
    return( dest );                             // destination string
}


void    st_ltrim( STRING string )
{
    register    char *src ;

    if ( strlen(string) )
    {
        src = string ;
        while ( isspace(*src) )
            src++ ;
        st_cpy( string, src, strlen(src)+1 );
    }
}

void    st_rtrim( STRING string )
{
    register    char *last ;

    last = string + strlen(string) ;            //  get to end of string
    while ( ((*last == '\0') || (isspace(*last))) && (last >= string) )
    {
        *last-- = '\0' ;
    }
}

void    st_trim( STRING string )
{
    st_rtrim( string );
    st_ltrim( string );
}


int     st_len( char str[] )
{
    register int i ;
    int l=0, x=strlen(str)-1;

    LOOP( i, 0, x, 1 )
    {
        if ( (str[i] != '\0') && (! isspace(str[i])) )
            l = i+1 ;
    }
    return( (int) l );
}


BOOL    st_eq( char *s1, char *s2 )
{
    if ( st_len(s1) != st_len(s2) )             // not equal length
        return( FALSE );
    if ( strnicmp( s1, s2, st_len(s1) ) == 0 )
        return( TRUE );
    else
        return( FALSE );
}


int     ut_color_code( const char sColor[] )
{
    int         colcode = -1 ;
    char        pcBuf2[3], pcBuf3[4], *color ;

    color = strdup( sColor );
    strlwr( color );
    switch ( color[0] )
    {
        case 'c' :                              // Cyan
            colcode = 3 ;
            break ;
        case 'm' :                              // Magenta
            colcode = 5 ;
            break ;
        case 'r' :                              // Red
            colcode = 4 ;
            break ;
        case 'w' :                              // White
            colcode = 7 ;
            break ;
        case 'y' :                              // Yellow
            colcode = 14 ;
            break ;
        case 'b' :
            switch ( color[1] )
            {
                case 'l' :
                    if ( color[2] == 'a' )      // BLAck
                        colcode = 0 ;
                    if ( color[2] == 'u' )      // BLUe
                        colcode = 1 ;
                    goto pc_done ;
                case 'r' :
                    if ( color[2] == 'o' )      // BROwn
                        colcode = 6 ;
                    if ( color[2] == 'w' )      // BRWhite
                        colcode = 15 ;
                    goto pc_done ;
                case 'w' :                      // BWhite
                    colcode = 15 ;
            }
            goto pc_done ;
        case 'g' :
            if ( color[1] == 'r' && color[2] == 'a' )  // GRAy
                colcode = 8 ;
            if ( color[1] == 'r' && color[2] == 'e' && color[3] == 'y' )  // GREY
                colcode = 8 ;
            if ( color[1] == 'r' && color[2] == 'e' && color[3] == 'e' )  // GREEn
                colcode = 2 ;
            break ;
    }
    ST_CPY( pcBuf2, color );                    // 2 chars
    ST_CPY( pcBuf3, color );                    // 3 chars
    if ( st_eq(pcBuf2, "lb") || st_eq(pcBuf3, "ltb") )  // light blue
    {
        colcode = 9 ;
        goto pc_done ;
    }
    if ( st_eq(pcBuf2, "lc") || st_eq(pcBuf3, "ltc") )  // light cyan
    {
        colcode = 11 ;
        goto pc_done ;
    }
    if ( st_eq(pcBuf2, "lg") || st_eq(pcBuf3, "ltg") )  // light green
    {
        colcode = 10 ;
        goto pc_done ;
    }
    if ( st_eq(pcBuf2, "lm") || st_eq(pcBuf3, "ltm") )  // light magenta
    {
        colcode = 13 ;
        goto pc_done ;
    }
    if ( st_eq(pcBuf2, "lr") || st_eq(pcBuf3, "ltr") )  // light red
    {
        colcode = 12 ;
        goto pc_done ;
    }
    if ( st_eq(pcBuf2, "lw") || st_eq(pcBuf3, "ltw") )  // bright white
    {
        colcode = 15 ;
        goto pc_done ;
    }

pc_done:
    free( color );
    return( colcode );
}


STRING  mth_name_( int n )                      // return the name of the month
{
    static STRING name[] =
    {
        "???",          // Illegal Month
        "January", "February", "March", "April", "May", "June",
        "July", "August", "September", "October", "November", "December"
    };
    return( name[n] );
}


static  int days_in_mth_[] = { 31,28,31,30,31,30,31,31,30,31,30,31 };
int   DaysInMonth( long ymd )
{
    int y, m, d ;

    if ( DateSplit( ymd, &y, &m, &d ) == -1 )
        return( 0 );                            // illegal date
    d = days_in_mth_[m-1] ;
    if ( m == 2 )                               // February
        if ( IsLeapYear( y ) )                  // leap year
            d = 29 ;
    return( d );
}


int   IsLeapYear( int year )
/*
        Determine whether year is a leap year.  If it is, return TRUE,
        otherwise return FALSE.
*/
{
    if ( year % 4 == 0 && year % 100 != 0 || year % 400 == 0 )
        return( TRUE );
    else
        return( FALSE );
}


int     DateSplit( long indate, int *yyyy, int *mm, int *dd )
/*
        Split indate into separate fields: yyyy, mm, and dd.

        Usage:      err = DateSplit( indate, &yyyy, &mm, &dd );

                    int     err     OK or -1
                    long    indate  date to be converted
                    int     *yyyy   the year, century default = 1900
                    int     *mm     the month
                    int     *dd     the day
*/
{
    if ( indate < 18000000L )
        indate = indate + 19000000L ;
    *dd     = (int) (indate % 100L) ;
    indate /= 100L ;
    *mm     = (int) (indate % 100L) ;
    *yyyy   = (int) (indate / 100L) ;
    if ( INRANGE(*yyyy,1800,2199) && INRANGE(*mm,1,12) && INRANGE(*dd,1,31) )
        return( 0 );
    *yyyy = 1980 ;
    *mm = *dd = 1 ;
    return( -1 );                               // illegal date
}


int   JulianDay( long ymd )
{
    int       i, y, m, d, days;

    if ( DateSplit( ymd, &y, &m, &d ) == -1 )
        return( 0 );                            // illegal date
    days = d;
    if ( m > 1 )
    {
        for ( i=1 ; i < m ; i++ )
        {
            days = days + DaysInMonth( ((long) y*10000L + i*100L + 1) );
        }
    }
    return( days );
}


long    Date2Num( long ymd )
/*
        Calculate a number for the given date.  This number represents the
        number of days elapsed since 31-Dec-1799, where 01-Jan-1800 = 1.
*/
{
    long       result=0;
    int       i, y, m, d, ref_year=1800;

    if ( DateSplit( ymd, &y, &m, &d ) == -1 )
        return( -1 );                           // illegal date
    if ( y < ref_year )
        return( -1 );                           // less than reference year
    d = JulianDay( ymd );
    result = d;                                 // make long
    if ( y > ref_year )
    {
        for ( i=ref_year ; i < y ; i++ )
        {
            result = result + 365L;
            if ( (IsLeapYear( i )) )
                result = result + 1L;
        }
    }
    return( result );
}


long   DaysBetween( long date1, long date2 )
{
    long       result, d1, d2 ;
    int       y, m, d ;

    if ( DateSplit( date1, &y, &m, &d ) == -1 )
        return( -1 );                           // illegal date
    if ( DateSplit( date2, &y, &m, &d ) == -1 )
        return( -1 );                           // illegal date
    d1 = Date2Num( date1 );
    d2 = Date2Num( date2 );
    if ( (d1 == -1) || (d2 == -1) )
        return( -1 );                           // illegal date
    result = d2 - d1;
    if ( result < 0 )
        result = -result;
    return( result );
}


int     dt_format_size( int outfmt )
{
    //  this table indicates the MAXIMUM length of dates formatted by FormatDate(),
    //  e.g., format code 0 returns a 7-char field, code 1 a 9-char field, etc.
    static int siz[] = { 7, 9, 11, 17, 18, 9, 14, 8, 3, 11, 12, 8, 8 };

    if ( OUTRANGE(outfmt,0,12) )
        return( -1 );                           // illegal format code
    return( siz[outfmt] );
}


int     FormatDate( long indate, STRING pdest, int outfmt )
/*
        Usage:  iret = FormatDate( indate, pdest, outfmt );

                int   iret    # of characters in destination string,
                                or -1 (illegal data or format codes)
                long   indate  date to be converted
                char   *pdest  pointer to output string
                int   outfmt  indicates how pdest is to be formatted:
                                0 = ddmmmyy
                                1 = dd-mmm-yy
                                2 = dd.mmm.yyyy
                                3 = dd month yyyy
                                4 = month dd, yyyy
                                5 = month
                                6 = month yyyy
                                7 = mmm-yyyy
                                8 = mmm
                                9 = dd-mmm-yyyy
                               10 = month dd
                               11 = mm/dd/yy
                               12 = yyyymmdd
*/
{
    char        day[4], month[11], year[6] ;
    int         yy, mm, dd, leng=4 ;

    *pdest = '\0' ;
    if ( OUTRANGE(outfmt,0,12) )
        return( -1 );                           // illegal format code
    if ( (outfmt > 2 && outfmt < 7) || outfmt == 10 )
        leng = 10 ;                             // full month name
    if ( DateSplit( indate, &yy, &mm, &dd ) == -1 )
        return( -1 );                           // illegal date
    if ( outfmt < 2 || outfmt == 11 )
    {
        yy = yy % 100 ;
        sprintf( year, "%02d", yy );            // 2 digit year
    }
    else
        sprintf( year, "%04d", yy );            // 4 digit year
    if ( outfmt == 4 )                          // month dd, yyyy
        sprintf( day, "%2d", dd );
    else
        sprintf( day, "%02d", dd );
    st_ltrim( day );
    if ( outfmt == 11 )                         // mm/dd/yy
    {
        sprintf( pdest, "%02d/%02d/%02d", mm, dd, yy );
        return( strlen(pdest) );
    }
    if ( outfmt == 12 )                         // yyyymmdd
    {
        sprintf( pdest, "%04d%02d%02d", yy, mm, dd );
        return( strlen(pdest) );
    }
    st_cpy( month, mth_name_(mm), leng );
    if ( outfmt == 4 )                          // month dd, yyyy
    {
        sprintf( pdest, "%s %s, %s", month, day, year );
        return( strlen(pdest) );
    }
    if ( outfmt == 10 )                         // month dd
    {
        sprintf( pdest, "%s %d", month, dd );
        return( strlen(pdest) );
    }
    if ( outfmt == 5 || outfmt == 8 )           // month only
    {
        st_cpy( pdest, month, 20 );
        return( strlen(pdest) );
    }

    if ( outfmt < 4 || outfmt == 9 )
        st_cpy( pdest, day, 20 );
    if ( outfmt == 1 || outfmt == 9 )           // dd-mmm-yy
        st_cat( pdest, "-", 20 );
    if ( outfmt == 2 )                          // dd.mmm.yyyy
        st_cat( pdest, ".", 20 );
    if ( outfmt == 3 )                          // dd month yyyy
        st_cat( pdest, " ", 20 );
    st_cat( pdest, month, 20 );
    if ( outfmt == 1 || outfmt == 7 || outfmt == 9 )
        st_cat( pdest, "-", 20 );               // dd-mmm-yy, mmm-yyyy, dd-mmm-yyyy
    if ( outfmt == 2 )                          // dd.mmm.yyyy
        st_cat( pdest, ".", 20 );
    if ( outfmt == 3 || outfmt == 6 )           // dd month yyyy or month yyyy
        st_cat( pdest, " ", 20 );
    st_cat( pdest, year, 20 );
    return( strlen(pdest) );
}


int     dt_getsystemdate( long *yyyymmdd, int *yyyy, int *mm, int *dd )
/*
//      Usage:  errcode = dt_getsystemdate(&ymd,&y,&m,&d);
*/
{
    int md, ye ;

    _asm
    {
        mov     ah, 2Ah
        int     21h
        mov     md, dx
        mov     ye, cx
    }

    *yyyy = ye ;
    *mm   = md >> 8 ;
    *dd   = md & 0xFF ;
    if ( OUTRANGE(*yyyy,1980,2099) || OUTRANGE(*mm,1,12) || OUTRANGE(*dd,1,31) )
    {
        *mm = *dd = 1;
        *yyyy  = 1980;
        *yyyymmdd = 19800101L;
        return( -1 );                           // illegal system date
    }
    *yyyymmdd = ((*yyyy)*10000L) + ((*mm)*100L) + (*dd);
    return( 0 );
}


long    dt_decode_str( char *datstr )           // decode date from "MM/DD/YY"
{                                               // returns -1 if invalid
    int         mon, day, year ;

    if ( sscanf( datstr, "%d/%d/%d", &mon, &day, &year ) == 3 )
        return( (year+1900)*10000L + mon*100L + day );
    return( -1 );
}


int     fl_parse_ini( STRING sFile, STRING sHeader, STRING sField, STRING sContent, int *iRecNo )
{
    FILE        *ini ;
    char        rec[256], *eq ;
    BOOL        looking4header = TRUE ;
    int         recct = 0 ;

    *sContent = '\0' ;                          // initialize
    *iRecNo = 0 ;
    ini = fopen( sFile, "r" );
    if ( ! ini )
        return( -1 );                           // can't open file

    while ( fgets( rec, sizeof(rec), ini ) )
    {
        recct++ ;
        st_trim( rec );
        if ( looking4header )
        {
            if ( rec[0] == '[' )
            {
                rec[0] = ' ' ;                  // remove [ and ]
                rec[strlen(rec)-1] = '\0' ;
                st_trim( rec );                 // trim again
                if ( st_eq(rec, sHeader) )
                    looking4header = FALSE ;    // found the proper header
            }
        }
        else                                    // we're looking for the field
        {
            if ( rec[0] == '[' )                // next header, so we missed it
            {
                fclose( ini );
                return( -1 );
            }
            //  see if we have the correct field
            eq = strchr( rec, '=' );
            if ( eq )
            {
                *eq = '\0' ;
                if ( st_eq(rec, sField) )
                {
                    eq++ ;                          // point beyond '='
                    strcpy( sContent, eq );
                    fclose( ini );
                    *iRecNo = recct ;
                    return( strlen(sContent) );     // we're done
                }
            }
        }
    }
    fclose( ini );
    return( -1 );                                   // not found
}


BOOL    fl_exist( STRING fname )
{
    FILE        *f ;

    f = fopen( fname, "r" );
    if ( ! f )
        return( FALSE );
    fclose( f );
    return( TRUE );
}


void    show_activity( long recno, int interval )
{
    static char symbol[]="|/-\\" ;
    static int  ctr = 0, maxsym=sizeof(symbol)-1 ;

    if ( recno % interval  )
        return ;
    fprintf( stderr, "\r%c", symbol[ctr] );
    ctr ++ ;
    if ( ctr >= maxsym )
        ctr = 0 ;
}


void    closing_comments( void )
{
    fprintf( stderr, "\nThis program has been donated to the public domain,\n"
                     "although all rights are reserved by GeCe Ventures.\n\n" );
}
